/*
 * files.c
 *
 * Copyright(c) 1999, Particle
 */

#include <stdio.h>
#include <stdlib.h>

#include "files.h"

#include "error.h"

/**
 * return the length of the file
 */
int fileLength(FILE *f){
	int	pos,end;
	pos = ftell(f);
	fseek(f,0,SEEK_END);
	end = ftell(f);
	fseek(f,pos,SEEK_SET);
	return end;
}

/**
 * open file for writing
 */
FILE* fileOpenWrite(char *filename){
	FILE* f;
	f = fopen(filename,"wb");
	if(!f)
		error("can't open file for writing: %s",filename);
	return f;
}

/**
 * open file for reading
 */
FILE* fileOpenRead(char *filename){
	FILE* f;
	f = fopen(filename, "rb");
	if(!f)
		error("can't open file for reading: %s",filename);
	return f;
}

/**
 * read the file
 */
int fileRead(FILE *f,void *buffer,int count){
	if(fread(buffer,1,count,f) != (size_t)count)
		error("file read failed");
	return 0;
}

/**
 * write file
 */
void fileWrite(FILE *f,void *buffer,int count){
	if(fwrite(buffer,1,count,f) != (size_t)count)
		error("file write failed");
}

/**
 * load file into memory (the memory pointer MUST be freed later)
 */
int fileLoad(char *filename,void **bufferptr){
	FILE	*f;
	int    length;
	void    *buffer;
	f = fileOpenRead(filename);
	length = fileLength(f);
	buffer = (char*)malloc(length+1);
	if(!buffer)
		error("error allocating %d bytes for file: %s",
		length+1,filename);
	((char *)buffer)[length] = 0;
	fileRead(f,buffer,length);
	fclose(f);
	*bufferptr = buffer;
	return length;
}
