/*
 * gen_asm_z.c
 *
 * Copyright(c) 1999, Particle
 */

#include <stdio.h>


#include "namevalue.h"

#include "parser_z.h"

#include "error.h"

#include "gen_asm_z.h"

void gen_asm_text(node_t* root,FILE* output){
	int l,k,m;
	if(root){
		switch(root->type){
		case NODE_READ:
			fprintf(output,"call getdec\n");
			fprintf(output,"mov  %s,ax\n",root->right->token->token);
			fprintf(output,"call newline\n");
			break;
		case NODE_WRITE:
			gen_asm_text(root->right,output);
			fprintf(output,"pop	ax\n");
			fprintf(output,"call printnum\n");
			fprintf(output,"call newline\n");
			break;

		case NODE_EQUALS:
			gen_asm_text(root->right,output);
			fprintf(output,"pop	ax\n");
			fprintf(output,"mov %s,ax\n",root->left->token->token);
			break;
		case NODE_ID:
			fprintf(output,"mov ax,%s\n",root->token->token);
			fprintf(output,"push ax\n");
			break;
		case NODE_LIT:
			fprintf(output,"mov ax,%s\n",root->token->token);
			fprintf(output,"push ax\n");
			break;
		case NODE_STATEMENT:
			while(root){
				gen_asm_text(root->right,output);
				root = root->next;
			}
			break;
		case NODE_ADD:

			gen_asm_text(root->left,output);
			gen_asm_text(root->right,output);
	
			fprintf(output,"pop ax\n");
			fprintf(output,"pop cx\n");
			fprintf(output,"add cx,ax\n");
			fprintf(output,"push cx\n");

			break;


		case NODE_SUB:

			gen_asm_text(root->left,output);
			gen_asm_text(root->right,output);

			fprintf(output,"pop ax\n");
			fprintf(output,"pop cx\n");
			fprintf(output,"sub cx,ax\n");
			fprintf(output,"push cx\n");

			break;

		case NODE_GT:

			gen_asm_text(root->left,output);
			gen_asm_text(root->right,output);

			fprintf(output,"pop ax\n");
			fprintf(output,"pop cx\n");
			fprintf(output,"cmp cx,ax\n");
			fprintf(output,"mov ax,0\n");
			fprintf(output,"jle l%d\n",l = rand());
			fprintf(output,"mov ax,1\n");
			fprintf(output,"l%d:\n",l);
			fprintf(output,"push ax\n");

			break;

		case NODE_LT:

			gen_asm_text(root->left,output);
			gen_asm_text(root->right,output);

			fprintf(output,"pop ax\n");
			fprintf(output,"pop cx\n");
			fprintf(output,"cmp cx,ax\n");
			fprintf(output,"mov ax,0\n");
			fprintf(output,"jge l%d\n",l = rand());
			fprintf(output,"mov ax,1\n");
			fprintf(output,"l%d:\n",l);
			fprintf(output,"push ax\n");

			break;


		case NODE_MULT:

			gen_asm_text(root->left,output);
			gen_asm_text(root->right,output);

			fprintf(output,"pop ax\n");
			fprintf(output,"pop cx\n");
			fprintf(output,"imul cx\n");
			fprintf(output,"push ax\n");

			break;

		case NODE_DIV:

			gen_asm_text(root->left,output);
			gen_asm_text(root->right,output);

			fprintf(output,"mov dx,0\n");
			fprintf(output,"pop cx\n");
			fprintf(output,"pop ax\n");
			fprintf(output,"idiv cx\n");
			fprintf(output,"push ax\n");
		
			break;		
		
		case NODE_BLOCK:
			gen_asm_text(root->right,output);

			break;
		case NODE_IF:


			gen_asm_text(root->next,output);
			fprintf(output,"pop	ax\n");
			fprintf(output,"cmp ax,0\n");
			
			fprintf(output,"jne l%d\n",m = rand());
			fprintf(output,"jmp l%d\n",l = rand());
			fprintf(output,"l%d:\n",m);

			gen_asm_text(root->left,output);
			
			if(root->right){    /* is there an else? */
				fprintf(output,"jmp l%d \n",k = rand());
				fprintf(output,"l%d: \n",l);
				gen_asm_text(root->right,output);
				fprintf(output,"l%d: \n",k);
			}else{
				fprintf(output,"l%d: \n",l);
			}
			
			break;
		case NODE_WHILE:
			
			fprintf(output,"l%d: \n",l = rand());	/* top of loop */
			gen_asm_text(root->next,output);
			fprintf(output,"pop	ax\n");
			fprintf(output,"cmp ax,0\n");
			fprintf(output,"jne l%d\n",m = rand());
			fprintf(output,"jmp l%d\n",k = rand());
			fprintf(output,"l%d:\n",m);

			gen_asm_text(root->left,output);
			
			fprintf(output,"jmp l%d\n",l);
			fprintf(output,"l%d:\n",k);
			
			break;

		}


	}
}

namevalue_t* traverse_to_find_datamembers(node_t* root,namevalue_t* namevalue){
	if(root){
		switch(root->type){
		case NODE_ID:
			namevalue = addUniqNameValue(namevalue,root->token->token,"");			
			break;
		case NODE_STATEMENT:
			while(root){
				namevalue = traverse_to_find_datamembers(root->right,namevalue);
				root = root->next;
			}
			break;
		case NODE_IF:
			namevalue = traverse_to_find_datamembers(root->next,namevalue);
		case NODE_READ:
		case NODE_WRITE:
		case NODE_EQUALS:
		case NODE_ADD:
		case NODE_SUB:
		case NODE_MULT:
		case NODE_DIV:
		case NODE_BLOCK:
		case NODE_WHILE:
			namevalue = traverse_to_find_datamembers(root->left,namevalue);
			namevalue = traverse_to_find_datamembers(root->right,namevalue);
		}
	}
	return namevalue;
}

void gen_asm_datamembers(node_t* root,FILE* output){
	namevalue_t* namevalue = 0;
	namevalue_t* p;
	namevalue = traverse_to_find_datamembers(root,namevalue);
	p=namevalue;
	while(p){
		fprintf(output,"%s dw 0\n",p->name);
		p = p->next;
	}
	freeNameValue(namevalue);
}

void gen_asm_header(node_t* root,FILE* output){
	
	fprintf(output,".model small\n");
	fprintf(output,".stack 100h\n");
	fprintf(output,".data\n");

	gen_asm_datamembers(root,output);

	fprintf(output,".code\n");
	fprintf(output,"extrn printnum:proc\n");
	fprintf(output,"extrn getdec:proc\n");
	fprintf(output,"extrn newline:proc\n");
	fprintf(output,"main proc\n");
	fprintf(output,"    mov ax,@data\n");
	fprintf(output,"    mov ds,ax\n");

}

void gen_asm_footer(node_t* root,FILE* output){
	
	fprintf(output,"    mov ax,4C00h\n");
	fprintf(output,"    int 21h\n");
	fprintf(output,"main endp\n");
	fprintf(output,"end main\n");

}

/*
 * generates asm given a parse tree
 */
void gen_asm(node_t* root,FILE* output){

	gen_asm_header(root,output);
	gen_asm_text(root,output);
	gen_asm_footer(root,output);

}
