/*
 * namevalue.c
 *
 * Copyright(c) 1999, Particle
 */

#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "namevalue.h"

/*
 * creates a new name value given a name and a value
 */
namevalue_t* createNameValue(char* name,char* value){
	namevalue_t* p;
	if(!name || !value)
		error("null pointer parameter to createNameValue()");

	p = (namevalue_t*)malloc(sizeof(namevalue_t));
	if(!p)
		error("cannot allocate %d bytes for name-value",sizeof(namevalue_t));
	p->name = (char*)malloc(strlen(name)+1);
	if(p->name == 0)
		error("cannot allocate memory for name in name value");
	p->value = (char*)malloc(strlen(value)+1);
	if(p->value == 0)
		error("cannot allocate memory for value in name value");
	strcpy(p->name,name);
	strcpy(p->value,value);
	p->next = 0;
	return p;
}

/*
 * adds a name & value to a list of name values. this method
 * can also be used to create name values by passing a null list
 */
namevalue_t* addNameValue(namevalue_t* l,char* name,char* value){
	namevalue_t* tmp = createNameValue(name,value);
	tmp->next = l;
	return tmp;
}

/*
 * this method makes sure that the name (in name value pair) is 
 * uniq in this namevalue list. it doesn't insert duplicates
 */
namevalue_t* addUniqNameValue(namevalue_t* l,char* name,char* value){
	if(!findNameValueByName(l,name))
		return addNameValue(l,name,value);
	return l;
}

/*
 * frees name values 
 */
void freeNameValue(namevalue_t* l){
	if(l){
		freeNameValue(l->next);
		free(l->name);
		free(l->value);
		free(l);
	}
}

/*
 * tries to find a name value by name; returns name 
 * value with that name or null if not found.
 */
namevalue_t* findNameValueByName(namevalue_t* l,char* name){
	if(l){
		if(!strcmp(l->name,name))
			return l;
		return findNameValueByName(l->next,name);		
	}
	return 0;
}

/*
 * tries to find a name value by value; returns name 
 * value with that value or null if not found.
 */
namevalue_t* findNameValueByValue(namevalue_t* l,char* value){
	if(l){
		if(!strcmp(l->value,value))
			return l;
		return findNameValueByValue(l->next,value);		
	}
	return 0;
}

