/*
 * nodes.c
 *
 * Copyright(c) 2000, Particle
 */

/*
    this module supports the parse tree
 */

#include <stdlib.h>

#include "nodes.h"

/*
 * creates a node given some parameters
 */
node_t* createNode(node_t* left,int type,node_t* right){
	node_t* node = (node_t*)malloc(sizeof(node_t));
	if(!node)
		error("cannot allocate %d bytes for node",sizeof(node_t));
	node->left = left;
	node->right = right;
	node->type = type;
	node->next = 0;
	return node;
}

node_t* createNameNode(char* s){
	node_t* node = createNode(0,NODE_NAME,0);
	node->u.val.sval = (char*)malloc(strlen(s)+1);
	strcpy(node->u.val.sval,s);
	return node;
}

node_t* createNumberNode(int n){
	node_t* node = createNode(0,NODE_NUMBER,0);
	node->u.val.nval = n;
	return node;
}

node_t* createIfNode(node_t* iftrue,node_t* iffalse,node_t* cond){
	node_t* node = createNode(iftrue,NODE_IF,iffalse);
	node->u.cond = cond;
	return node;
}

node_t* createWhileNode(node_t* iftrue,node_t* cond){
	node_t* node = createNode(0,NODE_WHILE,iftrue);
	node->u.cond = cond;
	return node;
}

/*
 * free node
 */
void freeNode(node_t* n){
	if(n){

		if(n->type == NODE_NAME)
			free(n->u.val.sval);
		else if(n->type == NODE_IF || n->type == NODE_WHILE)
			freeNode(n->u.cond);
		freeNode(n->left);
		freeNode(n->right);
		free(n);
	}
}


