/*
 * parser.h
 *
 * Copyright(c) 1999, Particle
 */

#ifndef _PARSER_H_
#define _PARSER_H_


#include "nodes.h"
#include "scanner.h"


typedef struct parser_s{
		/* the scanner */
	scanner_t* scanner;
		/* current token */
	token_t* token;
		
		/* various values defined in parser.c */
	int flags;
}parser_t;

/*
 * match function (for LL(1) parsers)
 *
 * recieves a parser (which contains current
 * token) and a string. If string matches current
 * token, scans in another token, and returns 1, 
 * otherwise returns zero (without scanning a new
 * token.
 */
int match(parser_t* parser,char* s);

/*
 * force match (gets rid of current token and gets
 * next one)
 */
void forceMatch(parser_t* parser);

/*
 * creates a parser given a scanner. The parser
 * doesn't take responsibility for the scanner
 * (you must free it yourself)
 */
parser_t* createParser(scanner_t* s);

/*
 * creates a parser from a file name. The localy
 * allocated scanner is freed automatically upon
 * destruction of this parser.
 */
parser_t* createParserFromFile(char* filename);

/*
 * frees a parser (and all related memory)
 *  uses the flags field of the parser to 
 * determine whether the scanner is locally
 * allocated or not.
 */
void freeParser(parser_t* p);


/*
 * external function to call this specific parser
 */
node_t* parse_program(parser_t* parser);


#endif _PARSER_H_