/*
 * parser_z.c
 *
 * Z language specific parser code
 *
 * a right derivation grammar
 *
 * Copyright(c) 1999, Particle
 */


#include "parser.h"
#include "parser_z.h"


node_t* parse_expr(parser_t* parser);
node_t* parse_factor(parser_t* parser);
node_t* parse_statementlist(parser_t* parser);

node_t* parse_id(parser_t* parser){
	node_t* p = 0;
	if(parser->token && parser->token->token){
		
		if(match(parser,"-"))
			parser->token->u.negative = 1;
		match(parser,"+");

		if(parser->token && parser->token->token){
		
			p = createNode(0,NODE_ID,0,parser->token);
			forceMatch(parser);
		}else{
			warning("%d: missing identifier",parser->scanner->lineno);
		}
	}else{
		warning("%d: missing identifier",parser->scanner->lineno);
	}
	return p;
}

node_t* parse_factortail(parser_t* parser){
	node_t* p = 0;
	
	if(parser->token == 0)
		return p;

	if(match(parser,"*")){
		node_t* tmp = 0;

		p = createNode(0,NODE_MULT,0,0);

		p->right = parse_factor(parser);

		tmp = parse_factortail(parser);

		if(tmp){
			tmp->left = p->right;
			p->right = tmp;
		}
	}else if(match(parser,"/")){
		node_t* tmp = 0;

		p = createNode(0,NODE_DIV,0,0);

		p->right = parse_factor(parser);

		tmp = parse_factortail(parser);

		if(tmp){
			tmp->left = p->right;
			p->right = tmp;
		}
	}

	return p;
}

node_t* parse_lit(parser_t* parser){
	node_t* p = 0;
	if(parser->token && parser->token->token){
		
		if(match(parser,"-"))
			parser->token->u.negative = 1;
		match(parser,"+");

		if(parser->token && parser->token->token){
			p = createNode(0,NODE_LIT,0,parser->token);
			forceMatch(parser);
		}else{
			warning("%d: missing literal",parser->scanner->lineno);
		}
	}else{
		warning("%d: missing literal",parser->scanner->lineno);
	}
	return p;
}

node_t* parse_factor(parser_t* parser){
	node_t* p = 0;

	if(match(parser,"(")){
		
		p = parse_expr(parser);
		
		match(parser,")");

	}else if(isdigit(parser->token->token[0])){
		p = parse_lit(parser);
	}else{
		p = parse_id(parser);
	}

	
	return p;
}

node_t* parse_term(parser_t* parser){
	node_t* p = 0;
	node_t* tmp = 0;	
	tmp = parse_factor(parser);
	p = parse_factortail(parser);
	if(p)
		p->left = tmp;
	else
		p = tmp;
	return p;
}

node_t* parse_termtail(parser_t* parser){
	node_t* p = 0;

	if(parser->token == 0)
		return p;

	if(match(parser,"+")){
		node_t* tmp = 0;

		p = createNode(0,NODE_ADD,0,0);

		p->right = parse_term(parser);

		tmp = parse_termtail(parser);

		if(tmp){
			tmp->left = p->right;
			p->right = tmp;
		}
	}else if(match(parser,"-")){
		
		node_t* tmp = 0;

		p = createNode(0,NODE_SUB,0,0);

		p->right = parse_term(parser);

		tmp = parse_termtail(parser);

		if(tmp){
			tmp->left = p->right;
			p->right = tmp;
		}

	}
	return p;
}



node_t* parse_comp_term(parser_t* parser){
	node_t* p = 0;
	node_t* tmp = 0;	
	tmp = parse_term(parser);
	p = parse_termtail(parser);
	if(p)
		p->left = tmp;
	else
		p = tmp;
	return p;
}

node_t* parse_comp_termtail(parser_t* parser){
	node_t* p = 0;
	if(parser->token == 0)
		return p;
	if(match(parser,"<")){
		node_t* tmp = 0;
		p = createNode(0,NODE_LT,0,0);
		p->right = parse_comp_term(parser);
		tmp = parse_comp_termtail(parser);
		if(tmp){
			tmp->left = p->right;
			p->right = tmp;
		}
	}else if(match(parser,">")){
		node_t* tmp = 0;
		p = createNode(0,NODE_GT,0,0);
		p->right = parse_comp_term(parser);
		tmp = parse_comp_termtail(parser);
		if(tmp){
			tmp->left = p->right;
			p->right = tmp;
		}
	}
	return p;
}




node_t* parse_expr(parser_t* parser){
	node_t* p = 0;
	node_t* tmp = 0;
	tmp = parse_comp_term(parser);
	p = parse_comp_termtail(parser);
	if(p)
		p->left = tmp;
	else
		p = tmp;
	return p;

}

node_t* parse_block(parser_t* parser){
	node_t* tmp = createNode(0,NODE_BLOCK,0,0);
	if(match(parser,"{"))
		tmp->right = parse_statementlist(parser);
	return tmp;
}

node_t* parse_ifstatement(parser_t* parser){
	node_t* p;
	node_t* tmp = parse_expr(parser);
	p = createNode(0,NODE_IF,0,0);
	p->next = tmp;	/* condition */
	p->left = parse_block(parser); /* true */
	if(match(parser,"else")){
		p->right = parse_block(parser); /* else */
	}
	return p;
}

node_t* parse_whilestatement(parser_t* parser){
	node_t* p;
	node_t* tmp = parse_expr(parser);
	p = createNode(0,NODE_WHILE,0,0);
	p->next = tmp;	/* condition */
	p->left = parse_block(parser); /* true */
	return p;
}

node_t* parse_statement(parser_t* parser){
	node_t* p = 0;
	p = createNode(0,NODE_STATEMENT,0,0);

	if(match(parser,"read")){
		node_t* tmp = parse_id(parser);
		p->right = createNode(0,NODE_READ,tmp,0);
	}else if(match(parser,"write")){
		node_t* tmp = parse_expr(parser);
		p->right = createNode(0,NODE_WRITE,tmp,0);
	}else if(match(parser,"while")){
		p->right = parse_whilestatement(parser);			
	}else if(match(parser,"if")){
		p->right = parse_ifstatement(parser);
	}else if(match(parser,"{")){
		p->right = parse_block(parser);
	}else{
		node_t* left,*right;
		left = parse_id(parser);

		if(!match(parser,"=")){
			error("error(%d): expecting '='",parser->scanner->lineno);
			forceMatch(parser);
		}
		
		right = parse_expr(parser);
		p->right = createNode(left,NODE_EQUALS,right,0);
	}
	return p;
}

node_t* parse_statementlist(parser_t* parser){
	
	node_t* p = 0;

	if(parser->token == 0)
		return p;

	if(match(parser,"}"))
		return p;
	p = parse_statement(parser);
	p->next = parse_statementlist(parser);
	return p;
}

/*
 * external function to call this specific parser
 */
node_t* parse_program(parser_t* parser){
	node_t* p;
	p = parse_statementlist(parser);
	return p;

}
