/*
 * scanner.h
 *
 * Copyright(c) 1999, Particle
 */

#ifndef _SCANNER_H_
#define _SCANNER_H_

#include <stdio.h>

typedef struct scanner_s{
	char* buffer;
	int where;
	int length;
	int lineno;
	int flags;
}scanner_t;

typedef struct token_s{
	char* token;
	int length;
	union{
		int negative;
	}u;
}token_t;

/*
 * creates a canner given a buffer and a length.
 * upon distruction of this canner, the library doesn't
 * free this buffer (in case it is used for other functions.
 */
scanner_t* createScanner(char* buffer,int length);

/*
 * creates a canner given a filename. the scanner
 * returned by this function will free the all allocated
 * memory upon distruction.
 */
scanner_t* createScannerFromFile(char* filename);

/*
 * resets scanner to it's initial condition.
 */
void resetScanner(scanner_t* p);

/*
 * free scanner. (and all related memory items)
 * this function must be called to ensure that all
 * memory has been freed. (simply calling free() on
 * the scanner is not recommended.
 */
void freeScanner(scanner_t* p);

/*
 * make a token given test and beginning and end
 */
token_t* makeToken(char* s,int start,int end);

/*
 * makes a token from a token (copy a token)
 */
token_t* makeTokenFromToken(token_t* t);

/*
 * free token
 */
void freeToken(token_t* p);

/* custom readToken for a language */
token_t* readToken(scanner_t* scanner);



#endif