%{

/*
 * z lexer
 *
 * Copyright(c) 2000, Particle
 */

#include <stdio.h>
#include <stdlib.h>

#include "nodes.h"

extern node_t* program;

/* apperently yacc makes code that uses this */
void* alloca(int n){
	return malloc(n);
}

%}

%token NAME NUMBER IF ELSE WHILE READ WRITE

%right '='
%nonassoc '>' '<'
%left '-' '+'
%left '*' '/'
%nonassoc UMINUS

%union{
	int nval;
	char *sval;
	node_t *node;
}

%type <node> expression statement statement_list statement_block program variable
%type <sval> NAME
%type <nval> NUMBER

%%
program:	statement_list	{ $$ = (program = createNode(0,NODE_PROGRAM,$1)); }
	;
statement_list: statement	{ $$ = createNode(0,NODE_LIST,$1); }
	| statement_list statement {
		node_t* n = $1;
		while(n->next)
			n = n->next;
		n->next = createNode(0,NODE_LIST,$2);
		$$ = $1;
		}
	;
statement_block: '{' statement_list '}' { $$ = createNode(0,NODE_BLOCK,$2); }
	;
statement: statement_block			
	| READ variable ';'				{ $$ = createNode(0,NODE_READ,$2); 
									($2)->type = NODE_DEREFNAME;  }
	| WRITE expression ';'		{ $$ = createNode(0,NODE_WRITE,$2); }
	| IF expression statement_block { $$ = createIfNode($3,0,$2); }
	| IF expression statement_block ELSE statement_block { $$ = createIfNode($3,$5,$2); }
	| WHILE expression statement_block { $$ = createWhileNode($3,$2); }
	| expression ';'
	;
expression:	variable '=' expression   { $$ = createNode($1,NODE_ASSIGN,$3); 
							($1)->type = NODE_DEREFNAME;	}
	|	expression '+' expression   { $$ = createNode($1,NODE_ADD,$3); }
	|	expression '>' expression   { $$ = createNode($1,NODE_GT,$3); }
	|	expression '<' expression   { $$ = createNode($1,NODE_LT,$3); }
	|	expression '-' expression   { $$ = createNode($1,NODE_SUB,$3); }
	|	expression '*' expression   { $$ = createNode($1,NODE_MUL,$3); }	
	|	expression '/' expression   { $$ = createNode($1,NODE_DIV,$3); }
	|	'-' expression %prec UMINUS { $$ = createNode(0,NODE_NEG,$2); }
	|	'(' expression ')' { $$ = $2; }
	|	NUMBER { $$ = createNumberNode($1); }
	|	variable
	;
variable: NAME { $$ = createNameNode($1); }
	;
%%

